/*
  $Id: extract_ddl.sql 22 2009-07-12 10:35:45Z dietmar.aust $

  Purpose  : Generate the DDL from the current Schema and store it into the table 
             XLIB_DDL_STATEMENTS
             
             Requirements: SELECT_CATALOG_ROLE   (grant SELECT_CATALOG_ROLE to <user>;)
                           ALTER USER <user> DEFAULT ROLE ALL;
                          
  $LastChangedDate: 2009-07-12 12:35:45 +0200 (So, 12. Jul 2009) $
  $LastChangedBy: dietmar.aust $ 

  Date        Author          Comment
  -------------------------------------------------------------------------------------------
  26.07.2008  D. Aust         Initial creation
  22.10.2008  D. Aust         included jobs from dbms_job (not included in dbms_metadata)

*/


TRUNCATE TABLE xlib_ddl_statements;

col CURR_USER new_value USR
select user CURR_USER from dual;

SET timing off

SET LINESIZE 132 PAGESIZE 0 FEEDBACK off VERIFY off TRIMSPOOL on LONG 1000000
set sqlblanklines on 
COLUMN ddl_string FORMAT A100 WORD_WRAP
COLUMN row_order FORMAT 999 NOPRINT

EXECUTE DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM,'STORAGE',false);
EXECUTE DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM,'PRETTY',true);
EXECUTE DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM,'SQLTERMINATOR',true);
EXECUTE DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM,'REF_CONSTRAINTS',false);
EXECUTE DBMS_METADATA.SET_TRANSFORM_PARAM(DBMS_METADATA.SESSION_TRANSFORM,'CONSTRAINTS_AS_ALTER', true);

alter session set NLS_NUMERIC_CHARACTERS='.,';

TRUNCATE TABLE xlib_ddl_statements;

------------------------------------------------------------------------------
PROMPT -- PROFILES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT PROFILE, 'PROFILE', 'PROFILE',
          DBMS_METADATA.get_ddl ('PROFILE', profile) ddl_string
     FROM (SELECT PROFILE
             FROM dba_users
            WHERE username = '&&USR'
              AND profile != 'DEFAULT') pr
/

------------------------------------------------------------------------------
PROMPT -- CREATE USER
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'USER', 'USER', 'USER', DBMS_METADATA.get_ddl ('USER', '&&USR') DDL
     FROM dual;

------------------------------------------------------------------------------
PROMPT -- DEFAULT_ROLE
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'DEFAULT_ROLE', 'DEFAULT_ROLE', 'USER',
          DBMS_METADATA.get_granted_ddl ('DEFAULT_ROLE', '&&USR') DDL
     FROM DUAL;

------------------------------------------------------------------------------
PROMPT -- TABLESPACE QUOTAS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'TABLESPACE_QUOTA', 'TABLESPACE_QUOTA', 'USER',
          DBMS_METADATA.get_granted_ddl ('TABLESPACE_QUOTA', username) DDL
     FROM (SELECT *
             FROM dba_users
            WHERE username = '&&USR'
              AND EXISTS (SELECT NULL
                            FROM dba_ts_quotas
                           WHERE username = UPPER ('&&USR')));


------------------------------------------------------------------------------
PROMPT -- ROLES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'ROLE_GRANT', 'ROLE_GRANT', 'USER',
          DBMS_METADATA.get_granted_ddl ('ROLE_GRANT', username) DDL
     FROM (SELECT *
             FROM dba_users
            WHERE username = '&&USR'
              AND EXISTS (SELECT NULL
                            FROM dba_role_privs
                           WHERE grantee = UPPER ('&&USR')));



------------------------------------------------------------------------------
PROMPT -- SYSTEM_GRANTS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'SYSTEM_GRANT', 'SYSTEM_GRANT', 'USER',
          DBMS_METADATA.get_granted_ddl ('SYSTEM_GRANT', username) DDL
     FROM (SELECT *
             FROM dba_users
            WHERE username = '&&USR'
              AND EXISTS (SELECT NULL
                            FROM dba_sys_privs
                           WHERE grantee = UPPER ('&&USR')));

------------------------------------------------------------------------------
PROMPT -- OBJECT_GRANTS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT 'OBJECT_GRANT', 'OBJECT_GRANT', 'OBJECT_GRANT',
          DBMS_METADATA.get_granted_ddl ('OBJECT_GRANT', username) DDL
     FROM (SELECT *
             FROM dba_users
            WHERE username = '&&USR'
              AND EXISTS (SELECT NULL
                            FROM dba_tab_privs
                           WHERE grantee = UPPER ('&&USR')));


------------------------------------------------------------------------------
PROMPT -- SEQUENCES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT sequence_name, 'SEQUENCE', sequence_name,
          DBMS_METADATA.get_ddl ('SEQUENCE',
                                 sequence_name,
                                 '&&USR'
                                ) ddl_string
     FROM user_sequences;


------------------------------------------------------------------------------
PROMPT -- DB_LINKS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT db_link, 'DB_LINK', 'DB_LINK',
          DBMS_METADATA.get_ddl ('DB_LINK', db_link, '&&USR') ddl_string
     FROM user_db_links;



------------------------------------------------------------------------------
PROMPT -- DIRECTORIES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT directory_name, 'DIRECTORY', 'DIRECTORY',
          DBMS_METADATA.get_ddl ('DIRECTORY',
                                 directory_name,
                                 '&&USR'
                                ) ddl_string
     FROM all_directories
    WHERE owner = '&&USR';

------------------------------------------------------------------------------
PROMPT -- TABLES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT table_name, 'TABLE', table_name,
          DBMS_METADATA.get_ddl ('TABLE', table_name, '&&USR') ddl_string
     FROM (SELECT table_name
             FROM user_tables where IOT_NAME is null
           MINUS
           SELECT mview_name
             FROM user_mviews);


------------------------------------------------------------------------------
PROMPT -- REF_CONSTRAINTS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT constraint_name, 'REF_CONSTRAINT', table_name,
             DBMS_METADATA.get_ddl ('REF_CONSTRAINT',
                                    constraint_name,
                                    '&&USR'
                                   ) ddl_string
--          || '/' ddl_string
     FROM user_constraints
    WHERE constraint_type = 'R';

------------------------------------------------------------------------------
PROMPT -- VIEWS
------------------------------------------------------------------------------
/*
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT view_name, 'VIEW', view_name,
          REPLACE (DBMS_METADATA.get_ddl ('VIEW', view_name, '&&USR'),
                   '",',
                   '", '
                  ) ddl_string
     FROM user_views;
*/

INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT view_name, 'VIEW', view_name, TO_LOB (text) ddl_string
     FROM user_views;

UPDATE xlib_ddl_statements
   SET ddl_text =
             'CREATE OR REPLACE FORCE VIEW '
          || ddl_object_name
          || '('
          || xlib_ddl_get_column_list(ddl_object_name)
          || ')'
          || ' AS '
          || CHR (10)
          || ddl_text
          || CHR (10)
          || '/'
 WHERE ddl_object_type = 'VIEW';

------------------------------------------------------------------------------
PROMPT -- FUNCTIONS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT NAME, 'FUNCTION', NAME,
          DBMS_METADATA.get_ddl ('FUNCTION', NAME, '&&USR') ddl_string
     FROM (SELECT DISTINCT NAME
                      FROM user_source
                     WHERE TYPE = 'FUNCTION');

------------------------------------------------------------------------------
PROMPT -- TRIGGERS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT trigger_name, 'TRIGGER', table_name,
          DBMS_METADATA.get_ddl ('TRIGGER', trigger_name, '&&USR') ddl_string
     FROM (SELECT trigger_name, table_name
             FROM user_triggers
            WHERE trigger_name NOT LIKE 'BIN$%');

------------------------------------------------------------------------------
PROMPT -- PACKAGES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT NAME, 'PACKAGE_SPEC', NAME,
          DBMS_METADATA.get_ddl ('PACKAGE_SPEC', NAME, '&&USR') ddl_string
     FROM (SELECT DISTINCT NAME
                      FROM user_source
                     WHERE TYPE = 'PACKAGE');

INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT NAME, 'PACKAGE_BODY', NAME,
          DBMS_METADATA.get_ddl ('PACKAGE_BODY', NAME, '&&USR') ddl_string
     FROM (SELECT DISTINCT NAME
                      FROM user_source
                     WHERE TYPE = 'PACKAGE BODY');

------------------------------------------------------------------------------
PROMPT -- PROCEDURES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT NAME, 'PROCEDURE', NAME,
          DBMS_METADATA.get_ddl ('PROCEDURE', NAME, '&&USR') ddl_string
     FROM (SELECT DISTINCT NAME
                      FROM user_source
                     WHERE TYPE = 'PROCEDURE');

------------------------------------------------------------------------------
PROMPT -- TYPES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT   type_name, 'TYPE_SPEC', type_name,
            DBMS_METADATA.get_ddl ('TYPE_SPEC', type_name,
                                   '&&USR') ddl_string
       FROM user_types
   ORDER BY DECODE (typecode, 'COLLECTION', 2, 1);

INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT   type_name, 'TYPE_BODY', type_name,
            DBMS_METADATA.get_ddl ('TYPE_BODY', type_name,
                                   '&&USR') ddl_string
       FROM user_types
      WHERE methods > 0
   ORDER BY DECODE (typecode, 'COLLECTION', 2, 1);


------------------------------------------------------------------------------
PROMPT -- SYNONYMS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT synonym_name, 'SYNONYM', synonym_name,
          DBMS_METADATA.get_ddl ('SYNONYM', synonym_name, '&&USR')
     FROM user_synonyms;


------------------------------------------------------------------------------
PROMPT -- MATERIALIZED_VIEWS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT mview_name, 'MATERIALIZED_VIEW', mview_name,
          DBMS_METADATA.get_ddl ('MATERIALIZED_VIEW',
                                 mview_name,
                                 '&&USR'
                                ) ddl_string
     FROM user_mviews;

INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT MASTER, 'MATERIALIZED_VIEW_LOG', MASTER,
          DBMS_METADATA.get_ddl ('MATERIALIZED_VIEW_LOG',
                                 log_table,
                                 '&&USR'
                                ) ddl_string
     FROM user_mview_logs;


------------------------------------------------------------------------------
PROMPT -- INDEXES
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements (ddl_object_name,
                                 ddl_object_type,
                                 ddl_base_object,
                                 ddl_text)
   SELECT   index_name,
            'INDEX',
            table_name,
            DBMS_METADATA.get_ddl ('INDEX', index_name, '&&USR') ddl_string
     FROM   (SELECT   index_name, table_name
               FROM   user_indexes
              WHERE   GENERATED = 'N' AND index_name NOT LIKE 'I_SNAP$%'
             MINUS
             SELECT   index_name, table_name FROM user_constraints);

------------------------------------------------------------------------------
PROMPT -- COMMENTS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT table_name, 'COMMENT', table_name,
          DBMS_METADATA.get_dependent_ddl ('COMMENT',
                                           table_name,
                                           '&&USR'
                                          ) ddl_string
     FROM (SELECT table_name
             FROM user_col_comments
            WHERE comments IS NOT NULL
           UNION
           SELECT table_name
             FROM user_tab_comments
            WHERE comments IS NOT NULL)
     WHERE   table_name not LIKE 'BIN$%';

------------------------------------------------------------------------------
PROMPT -- GRANTS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT table_name, 'GRANT', table_name,
          DBMS_METADATA.get_dependent_ddl ('OBJECT_GRANT',
                                           table_name,
                                           '&&USR'
                                          ) ddl_string
     FROM (SELECT DISTINCT table_name
                      FROM user_tab_privs_made);


------------------------------------------------------------------------------
PROMPT -- JOBS
------------------------------------------------------------------------------
INSERT INTO xlib_ddl_statements
            (ddl_object_name, ddl_object_type, ddl_base_object, ddl_text)
   SELECT job, 'JOB', 'JOB',
             'DECLARE'                                        || CHR (10)
          || '  l_job   NUMBER;'                              || CHR (10)
          || 'BEGIN'                                          || CHR (10)
          || '  DBMS_JOB.submit (job            => l_job,'    || CHR (10)
          || '                   what           => '''
          || replace(what, '''', '''''')
          || ''', '                                           || CHR (10)
          || '                   next_date      => to_date('''
          || TO_CHAR (next_date, 'dd.mm.yyyy hh24miss')
          || ''', ''dd.mm.yyyy hh24miss''),'                  || CHR (10)
          || '                   interval       => '''
          || INTERVAL
          || ''');'                                           || CHR (10)
          || 'END;'                                           || CHR (10)
          || '/'                                              || CHR (10)
          || CHR (10)
     FROM user_jobs;

                      
------------------------------------------------------------------------------
PROMPT -- REMOVE UNWANTED OBJECTS
------------------------------------------------------------------------------
DELETE FROM xlib_ddl_statements
      WHERE ddl_object_name LIKE '%XLIB_DDL%';
                      
COMMIT ;

EXIT;