/*
  $Id: spool_object_groups.sql 17 2009-05-20 10:28:30Z dietmar.aust $

  Purpose  : Spool the DDL from the table XLIB_DDL_STATEMENTS into the filesystem
             
  $LastChangedDate: 2009-05-20 12:28:30 +0200 (Mi, 20. Mai 2009) $
  $LastChangedBy: dietmar.aust $ 

  Date        Author          Comment
  ----------------------------------------------------------------------------------------
  26.07.2008  D. Aust         Initial creation
  14.10.2008  D. Aust         Better control over spooling multiple objects into a single file
                              lowercase filenames
  22.10.2008  D. Aust         included jobs from dbms_job (not included in dbms_metadata)
  17.08.2009  D. Aust         Trigger spool Files heissen <table>_trg.sql

*/

DEFINE EXPORT_DIR=build/export_ddl

set feedback off
set heading off
set verify off
set linesize 2000
set trimspool on
set pagesize 0
REM buffer overflow => reduze arraysize
set arraysize 2
set termout off
set timing off

-----------------------------------------------------------------------
-- create view XLIB_DDL_EXPORT_V 
------------------------------------------------------------------------
create or replace view XLIB_DDL_EXPORT_V
as
SELECT   /* RUN USER */
         CASE
            WHEN ddl_base_object IN ('USER', 'PROFILE', 'OBJECT_GRANT')
               THEN 'SYS'
            ELSE 'USER'
         END run_user,
         /* DIR */
         CASE
            WHEN ddl_object_type IN
                   ('PROFILE', 'USER', 'DEFAULT_ROLE', 'ROLE_GRANT',
                    'SYSTEM_GRANT', 'OBJECT_GRANT')
               THEN 'user'
            WHEN ddl_object_type IN ('SEQUENCE')
               THEN 'sequence'
            WHEN ddl_object_type IN ('DB_LINK')
               THEN 'db_link'
            WHEN ddl_object_type IN ('TABLE', 'REF_CONSTRAINT', 'INDEX', 'MATERIALIZED_VIEW')
               THEN 'table'
            WHEN ddl_object_type IN ('TRIGGER')
               THEN 'trigger'
            WHEN ddl_object_type IN ('VIEW')
               THEN 'view'
            WHEN ddl_object_type IN ('FUNCTION')
               THEN 'function'
            WHEN ddl_object_type IN ('PACKAGE_SPEC', 'PACKAGE_BODY')
               THEN 'package'
            WHEN ddl_object_type IN ('PROCEDURE')
               THEN 'procedure'
            WHEN ddl_object_type IN ('TYPE_SPEC', 'TYPE_BODY')
               THEN 'type'
            WHEN ddl_object_type IN ('SYNONYM')
               THEN 'synonym'
            WHEN ddl_object_type IN ('COMMENT')
               THEN 'comment'
            WHEN ddl_object_type IN ('GRANT')
               THEN 'grant'
            ELSE 'other'
         END dir,
         /* FILENAME */
         CASE
            WHEN ddl_object_type = 'REF_CONSTRAINT'
               THEN lower(ddl_base_object || '_FK.sql')            
            WHEN ddl_object_type = 'PACKAGE_SPEC'
               THEN lower(ddl_base_object || '.pks')            
            WHEN ddl_object_type = 'PACKAGE_BODY'
               THEN lower(ddl_base_object || '.pkb')            
            WHEN ddl_object_type = 'SEQUENCE'
               THEN lower('SEQUENCES.sql')            
            WHEN ddl_object_type = 'GRANT'
               THEN lower('GRANTS.sql')            
            WHEN ddl_object_type = 'SYNONYM'
               THEN lower('SYNONYMS.sql')            
            WHEN ddl_object_type = 'JOB'
               THEN lower('JOBS.sql')            
            WHEN ddl_object_type = 'TRIGGER'
               THEN lower(ddl_base_object || '_trg.sql')            
            ELSE lower(ddl_base_object || '.sql')
         END filename,
         ddl_object_name, ddl_object_type, ddl_base_object, ddl_text, ddl_id
		 , ddl_order_seq
    FROM xlib_ddl_statements
ORDER BY ddl_order_seq;


-----------------------------------------------------------------------
-- generate spool statements for all objects
------------------------------------------------------------------------
spool tmp_export.sql;

SELECT   '@@spool_object_group "&&EXPORT_DIR." "' || dir || '" "' || filename || '"'
    FROM xlib_ddl_export_v
GROUP BY dir, filename
order by min(ddl_order_seq);

spool off;

-----------------------------------------------------------------------
-- generate SYS run statement
------------------------------------------------------------------------
SPOOL &&EXPORT_DIR./run_sys.sql

select 'set sqlblanklines on' from dual;
select 'set scan off' from dual;
select 'set timing off' from dual;
select 'spool run_sys.log' from dual;

SELECT   'prompt *** ' || dir || '/' || filename || chr(10)
	||'@@"./' || dir || '/' || filename || '"'
    FROM xlib_ddl_export_v
   WHERE run_user = 'SYS'
GROUP BY dir, filename
order by min(ddl_order_seq);

select 'commit;' from dual;
select 'spool off' from dual;
select 'exit' from dual;

SPOOL off;

-----------------------------------------------------------------------
-- generate USER run statement
------------------------------------------------------------------------
SPOOL &&EXPORT_DIR./run_user.sql

select 'set sqlblanklines on' from dual;
select 'set scan off' from dual;
select 'set timing off' from dual;
select 'column object_name format a40;' from dual;
select 'column object_type format a40;' from dual;
select 'spool run_user.log' from dual;

SELECT   'prompt *** ' || dir || '/' || filename || chr(10)
	||'@@"./' || dir || '/' || filename || '"'
    FROM xlib_ddl_export_v
   WHERE run_user = 'USER'
GROUP BY dir, filename
order by min(ddl_order_seq);

select 'commit;' from dual;
select 'prompt *** recompile objects ***' from dual;
select 'EXEC DBMS_UTILITY.compile_schema(schema => user);' from dual;
select 'prompt *** invalid objects? ***' from dual;
select 'select object_name, object_type from user_objects where status = ''INVALID'';' from dual;

select 'host find "ORA-" *.log' from dual;
select 'host find "SP2-" *.log' from dual;

select 'spool off' from dual;
select 'exit' from dual;

SPOOL off;

-----------------------------------------------------------------------
-- spool DDL for all objects
------------------------------------------------------------------------
@@tmp_export.sql

-----------------------------------------------------------------------
-- drop view XLIB_DDL_EXPORT_V 
------------------------------------------------------------------------
--drop view xlib_ddl_export_v;

exit
