/*
  $Id: _install.sql 17 2009-05-20 10:28:30Z dietmar.aust $

  Purpose  : Install the required objects to support the extraction of the DDL
             
  $LastChangedDate: 2009-05-20 12:28:30 +0200 (Mi, 20. Mai 2009) $
  $LastChangedBy: dietmar.aust $ 

  Date        Version   Author          Comment
  -------------------------------------------------------------------------------------------
  26.07.2008  0.1.0.0   D. Aust         Initial creation

*/

create sequence xlib_seq;

/*
Prompt drop TABLE XLIB_DDL_STATEMENTS;
DROP TABLE XLIB_DDL_STATEMENTS CASCADE CONSTRAINTS
/
*/

Prompt Table XLIB_DDL_STATEMENTS;
--
-- XLIB_DDL_STATEMENTS  (Table) 
--
CREATE TABLE XLIB_DDL_STATEMENTS
(
  DDL_ID           NUMBER,
  DDL_OBJECT_NAME  VARCHAR2(200 CHAR),
  DDL_OBJECT_TYPE  VARCHAR2(200 CHAR),
  DDL_BASE_OBJECT  VARCHAR2(200 CHAR),
  DDL_TEXT         CLOB,
  DDL_ORDER_SEQ    NUMBER
)
/

Prompt Trigger DDL_BI_TRG;
--
-- XLIB_DDL_STATEMENTS_BI_TRG  (Trigger) 
--
CREATE OR REPLACE TRIGGER ddl_bi_trg
   BEFORE INSERT
   ON xlib_ddl_statements
   REFERENCING NEW AS NEW OLD AS OLD
   FOR EACH ROW
BEGIN
   IF :NEW.ddl_id IS NULL
   THEN
      SELECT xlib_seq.NEXTVAL
        INTO :NEW.ddl_id
        FROM DUAL;
      :new.ddl_order_seq := :new.ddl_id;
   END IF;
END xlib_ddl_statements_bi_trg;
/
SHOW ERRORS;


-- 
-- Non Foreign Key Constraints for Table XLIB_DDL_STATEMENTS 
-- 
Prompt Non-Foreign Key Constraints on Table XLIB_DDL_STATEMENTS;
ALTER TABLE XLIB_DDL_STATEMENTS ADD (
  CONSTRAINT DDL_PK
 PRIMARY KEY
 (DDL_ID))
/



CREATE OR REPLACE FUNCTION xlib_ddl_get_column_list (p_table_name varchar2)
   RETURN CLOB
IS
   l_columns   CLOB := EMPTY_CLOB;
   l_i         pls_integer := 0;
BEGIN
   FOR cur IN (  SELECT   table_name, column_name, column_id
                   FROM   user_tab_cols
                  WHERE   table_name = p_table_name
               ORDER BY   column_id)
   LOOP
      l_i := l_i + 1;

      IF l_i = 1
      THEN
         l_columns := cur.column_name;
      ELSE
         l_columns := l_columns || CHR (10) || ',' || cur.column_name;
      END IF;
   END LOOP;

   RETURN l_columns;
END;
/

